/*Copyright  2006 Altera Corporation. All rights reserved.  Altera products are
protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
other intellectual property laws.  

This reference design file, and your use thereof, is subject to and governed by
the terms and conditions of the applicable Altera Reference Design License Agreement.
By using this reference design file, you indicate your acceptance of such terms and
conditions between you and Altera Corporation.  In the event that you do not agree with
such terms and conditions, you may not use the reference design file. Please promptly
destroy any copies you have made.

This reference design file being provided on an "as-is" basis and as an accommodation 
and therefore all warranties, representations or guarantees of any kind 
(whether express, implied or statutory) including, without limitation, warranties of 
merchantability, non-infringement, or fitness for a particular purpose, are 
specifically disclaimed.  By making this reference design file available, Altera
expressly does not recommend, suggest or require that this reference design file be
used in combination with any other product not provided by Altera. */

#ifndef _FIR_H_
#define _FIR_H_


#include "system.h"

#define DATA_BUFFER_SIZE 200000

#if ((DATA_BUFFER_SIZE * 3) > (8 * 1024 * 1024))
  #error "Please reduce the buffer size otherwise you will run out of memory or the test may overflow the timer"
#endif

/* Modify FILTER_NUMBER to change the coefficients.  This system only supports
 * T = 15 or T = 16, to increase/decrease the tap length you must modified the
 * critical loop in "fir_hw.c". 
 * 
 * If you add your own filter coefficients add to the list below.  Be sure that
 * the tap length T is 15 or 16, the coefficients are symmetric, and that the
 * post scaling factor is a power of 2
 * Note:  DIVIDER_ORDER = log2(Post scale divider) */
 
/* 1 = 15_low_pass, 2 = 15_high_pass, 3 = 16_low_pass, 4 = 16_high_pass 
 * These filters use a hanning window a sampling rate of 1.0E7 and a cutoff
 * frequency of 1.25E6 */
#define FILTER_NUMBER 1

#if(FILTER_NUMBER == 1)
  #define DIVIDER_ORDER 15
  #define T 15
  short coefficients[T] = {
    #include "lpf_15_hanning.dat"
  };
#elif (FILTER_NUMBER == 2)
  #define DIVIDER_ORDER 13
  #define T 15  
  short coefficients[T] = {
    #include "hpf_15_hanning.dat"
  };
#elif (FILTER_NUMBER == 3)
  #define DIVIDER_ORDER 15
  #define T 16
  short coefficients[T] = {
    #include "lpf_16_hanning.dat"
  };
#elif (FILTER_NUMBER == 4)
  #define DIVIDER_ORDER 14
  #define T 16
  short coefficients[T] = {
    #include "hpf_16_hanning.dat"
  };
#else
  #error("You need to specify either 1, 2, 3, or 4 for the filter number")
#endif


/* Modify INPUT_DATA_SET to change the input data.  If you wish to add your
 * own input data set adjust the case statement below accordingly.
 * 
 * Input data set 1:
 * This input contains a sine wave at a high frequency and a constant value:
 * First sine:  Amplitude = 10000, Frequency = 4MHz
 * Second sine:  Amplitude = -33, Frequency = 0MHz (DC)
 * Length is 5 since that's how many samples are contain is a full period of both
 * 
 * Input data set 2:
 * This input contains two sine waves at high and low frequencies
 * First sine:  Amplitude = 100, Frequency = 2.5MHz
 * Second sine:  Amplitude = 1000, Frequency = 156.25kHz 
 * Length is 64 since that's how many samples are in the second sine wave */
#define INPUT_DATA_SET 1


#if (INPUT_DATA_SET == 1)
  #define INPUT_LENGTH 5
  short input_data[INPUT_LENGTH] = {
    #include "input1.dat"
  };
#elif (INPUT_DATA_SET == 2)
  #define INPUT_LENGTH 64
  short input_data[INPUT_LENGTH] = {
    #include "input2.dat"
  };
#else
  #error("You need to specify either 1, or 2 for the input data set")
#endif


void FIR_HW(short * __restrict__ input_data,
            long length,
            short * __restrict__ output_data,
            unsigned char tap_length,
            short coefA,
            short coefB,
            short coefC,
            short coefD,
            short coefE,
            short coefF,
            short coefG,
            short coefH,
            unsigned char divide_order);


void FIR_SW(short * __restrict__ input_data,
            long length,
            short * __restrict__ output_data,
            unsigned char tap_length,
            short coefA,
            short coefB,
            short coefC,
            short coefD,
            short coefE,
            short coefF,
            short coefG,
            short coefH,
            unsigned char divide_order);

#endif //_FIR_H_
