/*Copyright  2006 Altera Corporation. All rights reserved.  Altera products are
protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
other intellectual property laws.  

This reference design file, and your use thereof, is subject to and governed by
the terms and conditions of the applicable Altera Reference Design License Agreement.
By using this reference design file, you indicate your acceptance of such terms and
conditions between you and Altera Corporation.  In the event that you do not agree with
such terms and conditions, you may not use the reference design file. Please promptly
destroy any copies you have made.

This reference design file being provided on an "as-is" basis and as an accommodation 
and therefore all warranties, representations or guarantees of any kind 
(whether express, implied or statutory) including, without limitation, warranties of 
merchantability, non-infringement, or fitness for a particular purpose, are 
specifically disclaimed.  By making this reference design file available, Altera
expressly does not recommend, suggest or require that this reference design file be
used in combination with any other product not provided by Altera. */


#include "fir.h"
#include "stdio.h"
#include "sys/alt_alarm.h"

short source_databuffer[DATA_BUFFER_SIZE];
short sw_destination_databuffer[DATA_BUFFER_SIZE];
short hw_destination_databuffer[DATA_BUFFER_SIZE];


/* Author:  JCJB
 * Date:  08/02/2006
 * 
 * This software utilizes a C2H accelered 15 or 16 tap symmetric FIR filter.  The filter
 * supports reconfiguration of the coefficients given that they are symmetric.  This
 * example has four filters provided and allows for others to be easily added.
 * The following filters are provided:
 * 
 * All filters have a sample rate of 1.0E7 and cutoff frequency of 1.25E6
 * 1) low pass, hanning window, T = 15, coefficient width of 14 bits, divider of 2^15
 * 2) high pass, hanning window, T = 15, coefficient width of 14 bits, divider of 2^13
 * 3) low pass, hanning window, T = 16, coefficient width of 14 bits, divider of 2^15
 * 4) high pass, hanning window, T = 15, coefficient width of 14 bits, divider of 2^14
 * 
 * This example can be used with any low cost, standard, and full featured design however
 * it is recommended to use the full featured design.  It is also recommended to set the
 * application project and system library project to release mode.  This example uses the
 * high resolution timer so be sure to set that in the system libary properties page. 
 * 
 * All setting changes can be made within the file "fir.h" to change properites like the
 * filter type and input data set.  Carefully read the instructions provided in that file
 * to avoid causing the example to fail. */


/* Revision for DE2 by 4/2010 umb:
 * 1) add system clock in SOPC Builder to NIOS
 * 2) add LEDs
 * 3) add loop counter for C2H HW filter
 */
 
#define LEDs (char *) 0x01011030

int main()
{
  int i, loop;
  int failure;
  int sw_time_1, sw_time_2, hw_time_1, hw_time_2;
  
  
  printf("Creating buffer source data\n\n");
  for(i = 0; i < DATA_BUFFER_SIZE; i++)
  {
    source_databuffer[i] = input_data[i%INPUT_LENGTH];
  }


  
  printf("Software filter is operating (please be patient)\n");
  sw_time_1 = alt_nticks();
//  for (loop=0;loop<10;loop++){
//  *LEDs=loop;
  FIR_SW(source_databuffer,
         DATA_BUFFER_SIZE,
         sw_destination_databuffer,
         T,
         coefficients[0],
         coefficients[1],
         coefficients[2],
         coefficients[3],
         coefficients[4],
         coefficients[5],
         coefficients[6],
         coefficients[7],
         DIVIDER_ORDER);
  
//  }
  sw_time_2 = alt_nticks();       
  printf("Software filter is done\n\n");
  
  
  printf("Hardware filter is operating\n");
  hw_time_1 = alt_nticks();;

  for (loop=0;loop<40;loop++){
  *LEDs=loop>>3;  
  FIR_HW(source_databuffer,
         DATA_BUFFER_SIZE,
         hw_destination_databuffer,
         T,
         coefficients[0],
         coefficients[1],
         coefficients[2],
         coefficients[3],
         coefficients[4],
         coefficients[5],
         coefficients[6],
         coefficients[7],
         DIVIDER_ORDER);
  }
  hw_time_2 = alt_nticks();       
  printf("Hardware filter is done\n\n");


  printf("Comparing hardware and software filter results\n");
  i = 0;
  failure = 0;
  do
  {
    if(hw_destination_databuffer[i] != sw_destination_databuffer[i])
    {
      failure = 1;
    } 
  } while((failure != 1) && (++i < DATA_BUFFER_SIZE));
  
  if(failure == 0)
  {
    printf("The results match\n\n");
    printf("Processing times:\n");
  printf("Start SW ticks: %d \n", sw_time_1);
  printf("Finish SW ticks: %d \n", sw_time_2);
  printf("Start HW ticks: %d \n", hw_time_1);
  printf("Finish %d loops HW ticks: %d \n", loop, hw_time_2);
  printf("Ticks pers sec: %u\n", alt_ticks_per_second());
    printf("1 Loop SW processing time was: %f ms\n", ((double)1000*(sw_time_2-sw_time_1))/((double)alt_ticks_per_second()));
    printf("1 Loop HW processing time was: %f ms\n", ((double)1000/loop*(hw_time_2-hw_time_1))/((double)alt_ticks_per_second()));  
    printf("Hardware speedup factor over the software implementation was: %.2f times\n", ((double)loop*(sw_time_2-sw_time_1))/((double)(hw_time_2-hw_time_1)));
  }
  else
  {
    printf("The results do not match");
  }
  
  return 1; 
}
